/**
 * @file
 * VuoList_VuoMidiOutputDevice C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoMidiOutputDevice_H
#define VuoList_VuoMidiOutputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoMidiOutputDevice VuoList_VuoMidiOutputDevice
 * A list of @ref VuoMidiOutputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoMidiOutputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoMidiOutputDevice elements.
 */
typedef const struct VuoList_VuoMidiOutputDevice_struct { void *l; } * VuoList_VuoMidiOutputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoMidiOutputDevice elements.
 */
VuoList_VuoMidiOutputDevice VuoListCreate_VuoMidiOutputDevice(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoMidiOutputDevice VuoListCopy_VuoMidiOutputDevice(const VuoList_VuoMidiOutputDevice list);

/**
 * Returns the @ref VuoMidiOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoMidiOutputDevice VuoListGetValue_VuoMidiOutputDevice(const VuoList_VuoMidiOutputDevice list, const unsigned long index);

/**
 * Changes the @ref VuoMidiOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoMidiOutputDevice(const VuoList_VuoMidiOutputDevice list, const VuoMidiOutputDevice value, const unsigned long index);

/**
 * Inserts the `VuoMidiOutputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoMidiOutputDevice(const VuoList_VuoMidiOutputDevice list, const VuoMidiOutputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list, const VuoMidiOutputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list, const VuoMidiOutputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list);

/**
 * Removes the `VuoMidiOutputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoMidiOutputDevice(VuoList_VuoMidiOutputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoMidiOutputDevice(const VuoList_VuoMidiOutputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoMidiOutputDevice VuoList_VuoMidiOutputDevice_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoMidiOutputDevice_getJson(const VuoList_VuoMidiOutputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoMidiOutputDevice_getSummary(const VuoList_VuoMidiOutputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoMidiOutputDevice VuoList_VuoMidiOutputDevice_makeFromString(const char *str);
char * VuoList_VuoMidiOutputDevice_getString(const VuoList_VuoMidiOutputDevice value);
/// @}

/**
 * @}
 */

#endif
